@@ -645,7 +645,7 @@ pbm_print_page_loop(gx_device_printer * pdev, char magic, FILE * pstream,
                 code = gs_note_error(gs_error_ioerror);
                 goto punt;
             }
-            if (fprintf(pstream, "MAXVAL %d\n", pdev->color_info.max_gray) < 0) {
+            if (fprintf(pstream, "MAXVAL %d\n", 255) < 0) { /* force MAXVAL to 255 */
                 code = gs_note_error(gs_error_ioerror);
                 goto punt;
             }
@@ -661,8 +661,7 @@ pbm_print_page_loop(gx_device_printer * pdev, char magic, FILE * pstream,
                     goto punt;
                 }
             } else {
-                if (fprintf(pstream, "# Image generated by %s (device=%s)\n",
-                        gs_product, pdev->dname) < 0) {
+                if (fprintf(pstream, "# Image generated by %s\n", gs_product) < 0) {
                     code = gs_note_error(gs_error_ioerror);
                     goto punt;
                 }
@@ -1012,10 +1011,10 @@ pam4_print_row(gx_device_printer * pdev, byte * data, int depth,
             byte C = *data++;
             for (s = 7; s >= 0; s -= 4)
             {
-                fputc((C>>s    )&1, pstream);
-                fputc((C>>(s-1))&1, pstream);
-                fputc((C>>(s-2))&1, pstream);
-                fputc((C>>(s-3))&1, pstream);
+                fputc(((C>>s    )&1)*0xff, pstream);
+                fputc(((C>>(s-1))&1)*0xff, pstream);
+                fputc(((C>>(s-2))&1)*0xff, pstream);
+                fputc(((C>>(s-3))&1)*0xff, pstream);
                 w--;
                 if (w == 0)
                     break;
