@@ -30,6 +30,10 @@ typedef struct cmyk_color_s {
     frac k;
 } cmyk_color;
 
+/* if you make any additions/changes to the equivalent_cmyk_color_params or the
+   cmyk_color structrs you need to make the appropriate additions/changes 
+   to the compare_equivalent_cmyk_color_params() function in gdevdevn.c */
+
 /*
  * Structure for holding parameters for collecting the equivalent CMYK
  * for a spot colorant.
