@@ -1,6 +1,6 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zimage.c,v 1.2 2010/07/10 22:02:42 Arabidopsis Exp $ */
+/* $Id$ */
 /* Image operators */
 #include "math_.h"
 #include "memory_.h"
@@ -40,22 +40,20 @@
 
 /* Forward references */
 static int zimage_data_setup(i_ctx_t *i_ctx_p, const gs_pixel_image_t * pim,
-				 gx_image_enum_common_t * pie,
-				 const ref * sources, int npop);
+                                 gx_image_enum_common_t * pie,
+                                 const ref * sources, int npop);
 static int image_proc_process(i_ctx_t *);
 static int image_file_continue(i_ctx_t *);
 static int image_string_continue(i_ctx_t *);
 static int image_cleanup(i_ctx_t *);
 
-
-
 /* Extract and check the parameters for a gs_data_image_t. */
 int
-data_image_params(const gs_memory_t *mem, 
-		  const ref *op, gs_data_image_t *pim,
-		  image_params *pip, bool require_DataSource,
-		  int num_components, int max_bits_per_component,
-		  bool has_alpha)
+data_image_params(const gs_memory_t *mem,
+                  const ref *op, gs_data_image_t *pim,
+                  image_params *pip, bool require_DataSource,
+                  int num_components, int max_bits_per_component,
+                  bool has_alpha, bool islab)
 {
     int code;
     int decode_size;
@@ -64,95 +62,126 @@ data_image_params(const gs_memory_t *mem,
     check_type(*op, t_dictionary);
     check_dict_read(*op);
     if ((code = dict_int_param(op, "Width", 0, max_int_in_fixed / 2,
-			       -1, &pim->Width)) < 0 ||
-	(code = dict_int_param(op, "Height", 0, max_int_in_fixed / 2,
-			       -1, &pim->Height)) < 0 ||
-	(code = dict_matrix_param(mem, op, "ImageMatrix",
-				  &pim->ImageMatrix)) < 0 ||
-	(code = dict_bool_param(op, "MultipleDataSources", false,
-				&pip->MultipleDataSources)) < 0 ||
-	(code = dict_int_param(op, "BitsPerComponent", 1,
-			       max_bits_per_component, -1,
-			       &pim->BitsPerComponent)) < 0 ||
-	(code = decode_size = dict_floats_param(mem, op, "Decode",
-						num_components * 2,
-						&pim->Decode[0], NULL)) < 0 ||
-	(code = dict_bool_param(op, "Interpolate", false,
-				&pim->Interpolate)) < 0
-	)
-	return code;
+                               -1, &pim->Width)) < 0 ||
+        (code = dict_int_param(op, "Height", 0, max_int_in_fixed / 2,
+                               -1, &pim->Height)) < 0 ||
+        (code = dict_matrix_param(mem, op, "ImageMatrix",
+                                  &pim->ImageMatrix)) < 0 ||
+        (code = dict_bool_param(op, "MultipleDataSources", false,
+                                &pip->MultipleDataSources)) < 0 ||
+        (code = dict_int_param(op, "BitsPerComponent", 1,
+                               max_bits_per_component, -1,
+                               &pim->BitsPerComponent)) < 0 ||
+        (code = dict_bool_param(op, "Interpolate", false,
+                                &pim->Interpolate)) < 0
+        )
+        return code;
+
+    /* Decode size pulled out of here to catch case of Lab color space which
+       has a 4 entry range.  We also do NOT want to do Lab decoding IF range
+       is the common -128 127 for a and b. Otherwise we end up doing multiple
+       decode operations, since ICC flow will expect encoded data.
+       That is resolved later.  Also discovered that PDF write will stick
+       6 entry range in wich appears to be OK as far as AR is concerned so
+       we have to handle that too. */
+    if (islab) {
+        /* Note that it is possible that only the ab range values are there
+           or the lab values.  I have seen both cases.... */
+        code = decode_size = dict_floats_param(mem, op, "Decode", 4,
+                                                    &pim->Decode[2], NULL);
+        if (code < 0) {
+            /* Try for all three */
+            code = decode_size = dict_floats_param(mem, op, "Decode", 6,
+                                                        &pim->Decode[0], NULL);
+        } else {
+            /* Set the range on the L */
+            pim->Decode[0] = 0;
+            pim->Decode[1] = 100.0;
+        }
+        if (code < 0) return code;
+    } else {
+        code = decode_size = dict_floats_param(mem, op, "Decode",
+                                                    num_components * 2,
+                                                    &pim->Decode[0], NULL);
+        if (code < 0) return code;
+    }
     pip->pDecode = &pim->Decode[0];
     /* Extract and check the data sources. */
     if ((code = dict_find_string(op, "DataSource", &pds)) <= 0) {
-	if (require_DataSource)
-	    return (code < 0 ? code : gs_note_error(e_rangecheck));
-	return 1;		/* no data source */
+        if (require_DataSource)
+            return (code < 0 ? code : gs_note_error(e_rangecheck));
+        return 1;		/* no data source */
     }
     if (pip->MultipleDataSources) {
-	ref *ds = pip->DataSource;
+        ref *ds = pip->DataSource;
         long i, n = num_components + (has_alpha ? 1 : 0);
         if (!r_is_array(pds))
             return_error(e_typecheck);
-	if (r_size(pds) != n)
-	    return_error(e_rangecheck);
-	for (i = 0; i < n; ++i)
-	    array_get(mem, pds, i, &ds[i]);
+        if (r_size(pds) != n)
+            return_error(e_rangecheck);
+        for (i = 0; i < n; ++i)
+            array_get(mem, pds, i, &ds[i]);
         if (r_type(&ds[0]) == t_string) {
             /* We don't have a problem with the strings of different length
              * but Adobe does and CET tast 12-02.ps reports this as an error.
              */
-	    if (has_alpha)
+            if (has_alpha)
                 n--;
             for (i = 1; i < n; ++i) {
                 if (r_type(&ds[i]) == t_string && r_size(&ds[i]) != r_size(&ds[0])) {
-	            return_error(e_rangecheck);
+                    return_error(e_rangecheck);
                 }
             }
         }
     } else
-	pip->DataSource[0] = *pds;
+        pip->DataSource[0] = *pds;
     return 0;
 }
 
 /* Extract and check the parameters for a gs_pixel_image_t. */
 int
 pixel_image_params(i_ctx_t *i_ctx_p, const ref *op, gs_pixel_image_t *pim,
-		   image_params *pip, int max_bits_per_component,
-		   bool has_alpha, gs_color_space *csp)
+                   image_params *pip, int max_bits_per_component,
+                   bool has_alpha, gs_color_space *csp)
 {
+    bool islab = false;
     int num_components =
-	gs_color_space_num_components(csp);
+        gs_color_space_num_components(csp);
     int code;
 
     if (num_components < 1)
-	return_error(e_rangecheck);	/* Pattern space not allowed */
+        return_error(e_rangecheck);	/* Pattern space not allowed */
     pim->ColorSpace = csp;
+
+    if (pim->ColorSpace->cmm_icc_profile_data != NULL)
+        islab = pim->ColorSpace->cmm_icc_profile_data->islab;
+
     code = data_image_params(imemory, op, (gs_data_image_t *) pim, pip, true,
-			     num_components, max_bits_per_component,
-			     has_alpha);
+                             num_components, max_bits_per_component,
+                             has_alpha, islab);
     if (code < 0)
-	return code;
+        return code;
     pim->format =
-	(pip->MultipleDataSources ? gs_image_format_component_planar :
-	 gs_image_format_chunky);
+        (pip->MultipleDataSources ? gs_image_format_component_planar :
+         gs_image_format_chunky);
     return dict_bool_param(op, "CombineWithColor", false,
-			   &pim->CombineWithColor);
+                           &pim->CombineWithColor);
 }
 
 /* Common setup for all Level 1 and 2 images, and ImageType 4 images. */
 int
 zimage_setup(i_ctx_t *i_ctx_p, const gs_pixel_image_t * pim,
-	     const ref * sources, bool uses_color, int npop)
+             const ref * sources, bool uses_color, int npop)
 {
     gx_image_enum_common_t *pie;
     int code =
-	gs_image_begin_typed((const gs_image_common_t *)pim, igs,
-			     uses_color, &pie);
+        gs_image_begin_typed((const gs_image_common_t *)pim, igs,
+                             uses_color, &pie);
 
     if (code < 0)
-	return code;
+        return code;
     return zimage_data_setup(i_ctx_p, (const gs_pixel_image_t *)pim, pie,
-			     sources, npop);
+                             sources, npop);
 }
 
 /* Common code for .image1 and .alphaimage operators */
@@ -164,7 +193,6 @@ image1_setup(i_ctx_t * i_ctx_p, bool has_alpha)
     image_params    ip;
     int             code;
     gs_color_space *csp = gs_currentcolorspace(igs);
-    extern bool CPSI_mode;
 
     /* Adobe interpreters accept sampled images when the current color
      * space is a pattern color space using the base color space instead
@@ -172,7 +200,7 @@ image1_setup(i_ctx_t * i_ctx_p, bool has_alpha)
      * If all conditions are not met the pattern color space goes through
      * triggering a rangecheck error.
      */
-    if (CPSI_mode && gs_color_space_num_components(csp) < 1) {
+    if (gs_currentcpsimode(imemory) && gs_color_space_num_components(csp) < 1) {
        gs_color_space *bsp = csp->base_space;
        if (bsp)
          csp = bsp;
@@ -183,16 +211,16 @@ image1_setup(i_ctx_t * i_ctx_p, bool has_alpha)
                                op,
                                (gs_pixel_image_t *)&image,
                                &ip,
-			       (level2_enabled ? 16 : 8),
+                               (level2_enabled ? 16 : 8),
                                has_alpha, csp);
     if (code < 0)
-	return code;
+        return code;
 
     image.Alpha = (has_alpha ? gs_image_alpha_last : gs_image_alpha_none);
     return zimage_setup( i_ctx_p,
                          (gs_pixel_image_t *)&image,
                          &ip.DataSource[0],
-			 image.CombineWithColor,
+                         image.CombineWithColor,
                          1 );
 }
 
@@ -213,16 +241,15 @@ zimagemask1(i_ctx_t *i_ctx_p)
     int code;
 
     gs_image_t_init_mask_adjust(&image, false,
-				gs_incachedevice(igs) != CACHE_DEVICE_NONE);
+                                gs_incachedevice(igs) != CACHE_DEVICE_NONE);
     code = data_image_params(imemory, op, (gs_data_image_t *) & image,
-			     &ip, true, 1, 1, false);
+                             &ip, true, 1, 1, false, false);
     if (code < 0)
-	return code;
+        return code;
     return zimage_setup(i_ctx_p, (gs_pixel_image_t *)&image, &ip.DataSource[0],
-			true, 1);
+                        true, 1);
 }
 
-
 /* Common setup for all Level 1 and 2 images, and ImageType 3 and 4 images. */
 /*
  * We push the following on the estack.
@@ -256,7 +283,7 @@ zimagemask1(i_ctx_t *i_ctx_p)
 #define ETOP_NUM_SOURCES(ep) ((ep) - 1)
 static int
 zimage_data_setup(i_ctx_t *i_ctx_p, const gs_pixel_image_t * pim,
-		  gx_image_enum_common_t * pie, const ref * sources, int npop)
+                  gx_image_enum_common_t * pie, const ref * sources, int npop)
 {
     int num_sources = pie->num_planes;
     int inumpush = NUM_PUSH(num_sources);
@@ -282,61 +309,61 @@ zimage_data_setup(i_ctx_t *i_ctx_p, const gs_pixel_image_t * pim,
      * That is the reason for the aliasing information described above.
      */
     for (px = 0, pp = sources; px < num_sources; px++, pp++) {
-	es_ptr ep = EBOT_SOURCE(esp, px);
+        es_ptr ep = EBOT_SOURCE(esp, px);
 
-	make_int(ep + 1, 1);	/* default is no aliasing */
-	switch (r_type(pp)) {
-	    case t_file:
-		if (!level2_enabled)
-		    return_error(e_typecheck);
-		/* Check for aliasing. */
-		{
-		    int pi;
+        make_int(ep + 1, 1);	/* default is no aliasing */
+        switch (r_type(pp)) {
+            case t_file:
+                if (!level2_enabled)
+                    return_error(e_typecheck);
+                /* Check for aliasing. */
+                {
+                    int pi;
 
-		    for (pi = 0; pi < px; ++pi)
-			if (sources[pi].value.pfile == pp->value.pfile) {
-			    /* Record aliasing */
-			    make_int(ep + 1, -pi);
-			    EBOT_SOURCE(esp, pi)[1].value.intval++;
-			    break;
-			}
-		}
-		string_sources = false;
-		/* falls through */
-	    case t_string:
-		if (r_type(pp) != r_type(sources)) {
-		    gx_image_end(pie, false);    /* Clean up pie */
-		    return_error(e_typecheck);
-		}
-		check_read(*pp);
-		break;
-	    default:
-		if (!r_is_proc(sources)) {
-    		    static const char ds[] = "DataSource";
+                    for (pi = 0; pi < px; ++pi)
+                        if (sources[pi].value.pfile == pp->value.pfile) {
+                            /* Record aliasing */
+                            make_int(ep + 1, -pi);
+                            EBOT_SOURCE(esp, pi)[1].value.intval++;
+                            break;
+                        }
+                }
+                string_sources = false;
+                /* falls through */
+            case t_string:
+                if (r_type(pp) != r_type(sources)) {
+                    gx_image_end(pie, false);    /* Clean up pie */
+                    return_error(e_typecheck);
+                }
+                check_read(*pp);
+                break;
+            default:
+                if (!r_is_proc(sources)) {
+                    static const char ds[] = "DataSource";
                     gx_image_end(pie, false);    /* Clean up pie */
                     gs_errorinfo_put_pair(i_ctx_p, ds, sizeof(ds) - 1, pp);
-		    return_error(e_typecheck);
-		}
-		check_proc(*pp);
-		string_sources = false;
-	}
-	*ep = *pp;
+                    return_error(e_typecheck);
+                }
+                check_proc(*pp);
+                string_sources = false;
+        }
+        *ep = *pp;
     }
     /* Always place the image enumerator into local memory,
        because pie may have local objects inherited from igs,
-       which may be local when the current allocation mode is global. 
+       which may be local when the current allocation mode is global.
        Bug 688140. */
     if ((penum = gs_image_enum_alloc(imemory_local, "image_setup")) == 0)
-	return_error(e_VMerror);
+        return_error(e_VMerror);
     code = gs_image_enum_init(penum, pie, (const gs_data_image_t *)pim, igs);
     if (code != 0 || (pie->skipping && string_sources)) {		/* error, or empty image */
-	int code1 = gs_image_cleanup_and_free_enum(penum, igs);
+        int code1 = gs_image_cleanup_and_free_enum(penum, igs);
 
-	if (code >= 0)		/* empty image */
-	    pop(npop);
-	if (code >= 0 && code1 < 0)
-	    code = code1;
-	return code;
+        if (code >= 0)		/* empty image */
+            pop(npop);
+        if (code >= 0 && code1 < 0)
+            code = code1;
+        return code;
     }
     push_mark_estack(es_other, image_cleanup);
     esp += inumpush - 1;
@@ -344,15 +371,15 @@ zimage_data_setup(i_ctx_t *i_ctx_p, const gs_pixel_image_t * pim,
     make_int(ETOP_NUM_SOURCES(esp), num_sources);
     make_struct(esp, avm_local, penum);
     switch (r_type(sources)) {
-	case t_file:
-	    push_op_estack(image_file_continue);
-	    break;
-	case t_string:
-	    push_op_estack(image_string_continue);
-	    break;
-	default:		/* procedure */
-	    push_op_estack(image_proc_process);
-	    break;
+        case t_file:
+            push_op_estack(image_file_continue);
+            break;
+        case t_string:
+            push_op_estack(image_string_continue);
+            break;
+        default:		/* procedure */
+            push_op_estack(image_proc_process);
+            break;
     }
     pop(npop);
     return o_push_estack;
@@ -382,39 +409,39 @@ image_proc_continue(i_ctx_t *i_ctx_p)
     int i, code;
 
     if (!r_has_type_attrs(op, t_string, a_read)) {
-	check_op(1);
-	/* Procedure didn't return a (readable) string.  Quit. */
-	esp = zimage_pop_estack(esp);
-	image_cleanup(i_ctx_p);
-	return_error(!r_has_type(op, t_string) ? e_typecheck : e_invalidaccess);
+        check_op(1);
+        /* Procedure didn't return a (readable) string.  Quit. */
+        esp = zimage_pop_estack(esp);
+        image_cleanup(i_ctx_p);
+        return_error(!r_has_type(op, t_string) ? e_typecheck : e_invalidaccess);
     }
     size = r_size(op);
     if (size == 0 && ETOP_SOURCE(esp, 0)[1].value.intval == 0)
-	code = 1;
+        code = 1;
     else {
-	for (i = 0; i < num_sources; i++)
-	    plane_data[i].size = 0;
-	plane_data[px].data = op->value.bytes;
-	plane_data[px].size = size;
-	code = gs_image_next_planes(penum, plane_data, used);
-	if (code == e_RemapColor) {
-	    op->value.bytes += used[px]; /* skip used data */
-	    r_dec_size(op, used[px]);
-	    ETOP_SOURCE(esp, 0)[1].value.intval = 0; /* RemapColor callout */
-	    return code;
-	}
+        for (i = 0; i < num_sources; i++)
+            plane_data[i].size = 0;
+        plane_data[px].data = op->value.bytes;
+        plane_data[px].size = size;
+        code = gs_image_next_planes(penum, plane_data, used);
+        if (code == e_RemapColor) {
+            op->value.bytes += used[px]; /* skip used data */
+            r_dec_size(op, used[px]);
+            ETOP_SOURCE(esp, 0)[1].value.intval = 0; /* RemapColor callout */
+            return code;
+        }
     }
     if (code) {			/* Stop now. */
-	esp = zimage_pop_estack(esp);
-	pop(1);
-	image_cleanup(i_ctx_p);
-	return (code < 0 ? code : o_pop_estack);
+        esp = zimage_pop_estack(esp);
+        pop(1);
+        image_cleanup(i_ctx_p);
+        return (code < 0 ? code : o_pop_estack);
     }
     pop(1);
     wanted = gs_image_planes_wanted(penum);
     do {
-	if (++px == num_sources)
-	    px = 0;
+        if (++px == num_sources)
+            px = 0;
     } while (!wanted[px]);
     ETOP_PLANE_INDEX(esp)->value.intval = px;
     return image_proc_process(i_ctx_p);
@@ -430,9 +457,9 @@ image_proc_process(i_ctx_t *i_ctx_p)
 
     ETOP_SOURCE(esp, 0)[1].value.intval = 0; /* procedure callout */
     while (!wanted[px]) {
-	if (++px == num_sources)
-	    px = 0;
-	ETOP_PLANE_INDEX(esp)->value.intval = px;
+        if (++px == num_sources)
+            px = 0;
+        ETOP_PLANE_INDEX(esp)->value.intval = px;
     }
     pp = ETOP_SOURCE(esp, px);
     push_op_estack(image_proc_continue);
@@ -448,91 +475,94 @@ image_file_continue(i_ctx_t *i_ctx_p)
     int num_sources = ETOP_NUM_SOURCES(esp)->value.intval;
 
     for (;;) {
-	uint min_avail = max_int;
-	gs_const_string plane_data[GS_IMAGE_MAX_COMPONENTS];
-	int code;
-	int px;
-	const ref *pp;
-	int at_eof_count = 0;
+        uint min_avail = max_int;
+        gs_const_string plane_data[GS_IMAGE_MAX_COMPONENTS];
+        int code;
+        int px;
+        const ref *pp;
+        int at_eof_count = 0;
+        int total_used;
 
-	/*
-	 * Do a first pass through the files to ensure that at least
-	 * one has data available in its buffer.
-	 */
+        /*
+         * Do a first pass through the files to ensure that at least
+         * one has data available in its buffer.
+         */
 
-	for (px = 0, pp = ETOP_SOURCE(esp, 0); px < num_sources;
-	     ++px, pp -= 2
-	    ) {
-	    int num_aliases = pp[1].value.intval;
-	    stream *s = pp->value.pfile;
-	    int min_left;
-	    uint avail;
+        for (px = 0, pp = ETOP_SOURCE(esp, 0); px < num_sources;
+             ++px, pp -= 2
+            ) {
+            int num_aliases = pp[1].value.intval;
+            stream *s = pp->value.pfile;
+            int min_left;
+            uint avail;
 
-	    if (num_aliases <= 0)
-		num_aliases = ETOP_SOURCE(esp, -num_aliases)[1].value.intval;
-	    while ((avail = sbufavailable(s)) <=
-		   (min_left = sbuf_min_left(s)) + num_aliases - 1) {
-		int next = s->end_status;
+            if (num_aliases <= 0)
+                num_aliases = ETOP_SOURCE(esp, -num_aliases)[1].value.intval;
+            while ((avail = sbufavailable(s)) <=
+                   (min_left = sbuf_min_left(s)) + num_aliases - 1) {
+                int next = s->end_status;
 
-		switch (next) {
-		case 0:
-		    s_process_read_buf(s);
-		    continue;
-		case EOFC:
-		    at_eof_count++;
-		    break;	/* with no data available */
-		case INTC:
-		case CALLC:
-		    return
-			s_handle_read_exception(i_ctx_p, next, pp,
-						NULL, 0, image_file_continue);
-		default:
-		    /* case ERRC: */
-		    return_error(e_ioerror);
-		}
-		break;		/* for EOFC */
-	    }
-	    /*
-	     * Note that in the EOF case, we can get here with no data
-	     * available.
-	     */
-	    if (avail >= min_left)
-		avail = (avail - min_left) / num_aliases; /* may be 0 */
-	    if (avail < min_avail)
-		min_avail = avail;
-	    plane_data[px].data = sbufptr(s);
-	    plane_data[px].size = avail;
-	}
+                switch (next) {
+                case 0:
+                    s_process_read_buf(s);
+                    continue;
+                case EOFC:
+                    at_eof_count++;
+                    break;	/* with no data available */
+                case INTC:
+                case CALLC:
+                    return
+                        s_handle_read_exception(i_ctx_p, next, pp,
+                                                NULL, 0, image_file_continue);
+                default:
+                    /* case ERRC: */
+                    return_error(e_ioerror);
+                }
+                break;		/* for EOFC */
+            }
+            /*
+             * Note that in the EOF case, we can get here with no data
+             * available.
+             */
+            if (avail >= min_left)
+                avail = (avail - min_left) / num_aliases; /* may be 0 */
+            if (avail < min_avail)
+                min_avail = avail;
+            plane_data[px].data = sbufptr(s);
+            plane_data[px].size = avail;
+        }
 
-	/*
-	 * Now pass the available buffered data to the image processor.
-	 * Even if there is no available data, we must call
-	 * gs_image_next_planes one more time to finish processing any
-	 * retained data.
-	 */
+        /*
+         * Now pass the available buffered data to the image processor.
+         * Even if there is no available data, we must call
+         * gs_image_next_planes one more time to finish processing any
+         * retained data.
+         */
 
-	{
-	    int pi;
-	    uint used[GS_IMAGE_MAX_COMPONENTS];
+        {
+            int pi;
+            uint used[GS_IMAGE_MAX_COMPONENTS];
 
-	    code = gs_image_next_planes(penum, plane_data, used);
-	    /* Now that used has been set, update the streams. */
-	    for (pi = 0, pp = ETOP_SOURCE(esp, 0); pi < num_sources;
-		 ++pi, pp -= 2
-		 )
-		sbufskip(pp->value.pfile, used[pi]);
-	    if (code == e_RemapColor)
-		return code;
-	}
-	if (at_eof_count >= num_sources)
-	    code = 1;
-	if (code) {
-	    int code1;
+            code = gs_image_next_planes(penum, plane_data, used);
+            /* Now that used has been set, update the streams. */
+            total_used = 0;
+            for (pi = 0, pp = ETOP_SOURCE(esp, 0); pi < num_sources;
+                 ++pi, pp -= 2 ) {
+                sbufskip(pp->value.pfile, used[pi]);
+                total_used += used[pi];
+            }
+            if (code == e_RemapColor)
+                return code;
+        }
+        if (at_eof_count >= num_sources || (at_eof_count && total_used == 0))
+            code = 1;
+        if (code) {
+            int code1;
 
-	    esp = zimage_pop_estack(esp);
-	    code1 = image_cleanup(i_ctx_p);
-	    return (code < 0 ? code : code1 < 0 ? code1 : o_pop_estack);
-	}
+            esp = zimage_pop_estack(esp);
+            code1 = image_cleanup(i_ctx_p);
+            return (code < 0 ? code : code1 < 0 ? code1 : o_pop_estack);
+        }
     }
 }
 
@@ -549,29 +579,29 @@ image_string_continue(i_ctx_t *i_ctx_p)
     /* Pass no data initially, to find out how much is retained. */
     memset(sources, 0, sizeof(sources[0]) * num_sources);
     for (;;) {
-	int px;
-	int code = gs_image_next_planes(penum, sources, used);
+        int px;
+        int code = gs_image_next_planes(penum, sources, used);
 
-	if (code == e_RemapColor)
-	    return code;
+        if (code == e_RemapColor)
+            return code;
     stop_now:
-	if (code) {		/* Stop now. */
-	    esp -= NUM_PUSH(num_sources);
-	    image_cleanup(i_ctx_p);
-	    return (code < 0 ? code : o_pop_estack);
-	}
-	for (px = 0; px < num_sources; ++px)
-	    if (sources[px].size == 0) {
-		const ref *psrc = ETOP_SOURCE(esp, px);
-		uint size = r_size(psrc);
+        if (code) {		/* Stop now. */
+            esp -= NUM_PUSH(num_sources);
+            image_cleanup(i_ctx_p);
+            return (code < 0 ? code : o_pop_estack);
+        }
+        for (px = 0; px < num_sources; ++px)
+            if (sources[px].size == 0) {
+                const ref *psrc = ETOP_SOURCE(esp, px);
+                uint size = r_size(psrc);
 
-		if (size == 0) {	    /* empty source */
-		    code = 1;
-		    goto stop_now;
+                if (size == 0) {	    /* empty source */
+                    code = 1;
+                    goto stop_now;
                 }
-		sources[px].data = psrc->value.bytes;
-		sources[px].size = size;
-	    }
+                sources[px].data = psrc->value.bytes;
+                sources[px].size = size;
+            }
     }
 }
 
@@ -581,7 +611,7 @@ image_cleanup(i_ctx_t *i_ctx_p)
 {
     es_ptr ep_top = esp + NUM_PUSH(EBOT_NUM_SOURCES(esp)->value.intval);
     gs_image_enum *penum = r_ptr(ep_top, gs_image_enum);
-    
+
     return gs_image_cleanup_and_free_enum(penum, igs);
 }
 
@@ -591,7 +621,7 @@ const op_def zimage_op_defs[] =
 {
     {"1.image1", zimage1},
     {"1.imagemask1", zimagemask1},
-		/* Internal operators */
+                /* Internal operators */
     {"1%image_proc_continue", image_proc_continue},
     {"0%image_file_continue", image_file_continue},
     {"0%image_string_continue", image_string_continue},
